/*** CALCULETTE PROGRAMMABLE (style Maple):
   Syntaxe LALR(1)
   Evaluation par un (doublet) attribut synthetise $
   auteur : PFZ
   version origine: Mars 1999 avec l'aide de J. Farre'
   nouvelle version: Mars 2000 avec l'aide de T. Graf
   nouvelle version 2001: 
 
Versions et MODIFS pour la (super)Calculette
_______________________________________________

pour 2000:
- ajout de la dcompilation du source
- nouveau makefile
- nouvelle version de l'affichage et dialogue

pour 2001: 
- grammaire E+E avec priorit
- accepter les Var avec Majuscules (transposes en minuscules par lex)

pour 2002:
- utiliser la grammaire ETF + Primaire pour - unaire et ^ 
- type abstrait pour la memoire REGISTRES avec setV et getV
- revoir le rcupration d'erreurs, insuffisante pour les regles basses de la grammaire
- decompilation via HTML ?
- IHM via java ?

*********************************************************/

/* declarations YACC *************/
%union {          /* le doublet d'attributs YACC */
    double val ;
    char let ;
}
%token <val> ENT DERNIER
%token <let> VAR
%token FINDELIGNE CLEAR RESET STOP SUM FROM TO BY SINUS COSINUS PI
%type <val> exp cond
%left '+' '-'
%left '*' '/'
%left '^'
%left UMOINS
%{
#include <math.h> /* ajouter -lm au link */
#include <stdio.h>

#include "affichage.h"

#include "memoire.h"

%}
/***** DEBUT des actions YACC ****/
%%
calc   : ligne
       | calc ligne
       ;

ligne  : error  FINDELIGNE
		{yyerrok; lig -= 1;
                  Mon_yyerror ("erreur de syntaxe");}
       | FINDELIGNE
		{
                aff_dialogue("?");
		}
       |  exp FINDELIGNE
		{
		aff_exp($1);
		dernier=$1;
		}
       |  VAR '=' exp FINDELIGNE
                {
                aff_affect($1, $3); aff_une_variable($1, $3);

                table[$1 - 'a'] = $3 ;
		dernier=$3;
                }
       |  CLEAR VAR FINDELIGNE
	        {
		aff_clear($2); aff_une_variable($2, 0);

                table[$2 - 'a'] = 0 ;
                }
       |  RESET FINDELIGNE
	        {
		 aff_dialogue("reset des variables.....");
		 fflush(stdout); 
                 for (i=0; i<26; i++) table[i]=0 ;
		 aff_les_variables();
                 sleep(2);
		 prompteur();

                }
       |  STOP  FINDELIGNE
                {
		 aff_dialogue(" Au revoir\n");

                YYACCEPT ;}
       ;

exp    :  exp '+' exp
		             {$$ = $1 + $3 ;}
       |  exp '-' exp
		             {$$ = $1 - $3 ;}
       |  exp '*' exp
		             {$$ = $1 * $3 ;}
       |  exp '/' exp
                             { if ($3 == 00) {
	                             Mon_yyerror("division par zero");
			             $$ = 999999;
			          } else
			          $$ = $1 / $3 ;
			     }
       | '(' exp ')'
                        	{$$ = $2 ;}
       | exp '^' exp
                        	{$$ = pow($1, $3) ;}
       | '-' exp %prec UMOINS
                        	{$$ = -$2 ;}
       | ENT
                                {$$ = $1 ;}   
       | DERNIER
                                {
				  $$ = dernier ;
				}

       | PI
                                {$$ = 3.14159 ;}
       | VAR
                                {$$ = table[$1-'a'] ;}
       | '(' cond '?' exp ':' exp  ')'
                        	{
				  $$ = ($2==0)?$4:$6;
				}
       | '(' FROM exp TO exp SUM exp ')'
                        	{
				  S=0;
				  for (i= $3 ; i<=$5 ; i+=1) S+= $7;
				  $$= S;
				}
       |  '(' FROM exp TO exp BY exp SUM exp ')'
                          {
			    S=0;
			    for (i= $3 ; i<=$5 ; i+=$7) S+= $9;
			    $$ = S;
			  }       
       | SINUS '(' exp ')'
                        	{$$ = Eval("sin", $3 );}
       | COSINUS '(' exp ')'
                        	{$$ = Eval("cos", $3 );}
       ;


cond   :  exp '>' exp
		             {$$ = ($1 > $3)? 0:1;}
       |  exp '>' '=' exp
		             {$$ = ($1 >= $4)? 0:1;}
       |  exp '<' exp
		             {$$ = ($1 <  $3)? 0:1;}
       |  exp '<' '=' exp
		             {$$ = ($1 <= $4)? 0:1;}
       |  exp '=' '=' exp
		             {$$ = ($1 == $4)? 0:1;}
       |  exp '!' '=' exp
		             {$$ = ($1 != $4)? 0:1;}
       ;
%%

extern int lig;

int yyparse();

int yyerror(char *s)   /* pour masquer les messages de yacc */
{
  /* printf("ligne %d : %s\n",lig,s); */
  return 1;
}

int Mon_yyerror(char *s) /* pour masquer celui de yacc */
{
  aff_erreur(lig, s);
  return 1;
}

int main() {
  dessine_calculette();

  return yyparse();
}

