/*** CALCULETTE PROGRAMMABLE (style Maple):
   Syntaxe LALR(1)
   Evaluation par un (triplet) attribut synthetise $
   auteur : PFZ
   version origine: Mars 1999 avec l'aide de J. Farre'
   nouvelle version: Mars 2000 avec l'aide de T. Graf
   nouvelle version 2001: 
 
Versions et MODIFS pour la (super)Calculette
_______________________________________________

pour 2000:
- ajout de la dcompilation du source
- nouveau makefile
- nouvelle version de l'affichage et dialogue

pour 2001: 
- grammaire E+E avec priorit
- accepter les Var avec Majuscules (transposes en minuscules par lex)

pour 2002:
- utiliser la grammaire ETF + Primaire pour - unaire et ^ 
- type abstrait pour la memoire REGISTRES avec setV et getV
- revoir le rcupration d'erreurs, insuffisante pour les regles basses de la grammaire
- decompilation via HTML ?
- IHM via java ?
*********************************************************/

/* declarations YACC *************/
%union {          /* le triplet d'attributs YACC */
    struct A {
      double val ;
      char let ;
      char* text ;
    } att;
}
%token <att> ENT VAR DERNIER
%token FINDELIGNE CLEAR RESET STOP SUM FROM TO BY SINUS COSINUS PI 
%type <att> exp cond ligne calc
%left '+' '-'
%left '*' '/'
%left '^'
%left UMOINS
%{

#include <math.h> /* ajouter -lm au link */
#include <stdio.h>
#include "schemas.h" /* SCHEMAS d"ATTRIBUTION POUR LES REGLES DE YACC ***/
#include "affichage.h"

#include "memoire.h"

%}
%%
calc   : ligne
                {
		  aff_source(lig, $1.text);
		}
       | calc ligne
                {
		  aff_source(lig, $2.text);
		}
       ;

ligne  : error  FINDELIGNE
		{yyerrok; lig -= 1;
                  Mon_yyerror ("erreur de syntaxe");}
       | FINDELIGNE
		{
                aff_dialogue("?");
		}
       |  exp FINDELIGNE
		{
		aff_exp($1.val);
                dernier=$1.val;
		$$.text = $1.text;
                
		}
       |  VAR '=' exp FINDELIGNE
                {
                aff_affect($1.let, $3.val); aff_une_variable($1.let, $3.val);
                table[$1.let - 'a'] = $3.val ;
                dernier=$3.val;
		DECOMP_AFF($$, $1, $3);

                }
       |  CLEAR VAR FINDELIGNE
	        {
		aff_clear($2.let); aff_une_variable($2.let, 0);
                table[$2.let - 'a'] = 0 ;
		DECOMP_CLEAR($$, $2);
                }
       |  RESET FINDELIGNE
	        {
		 aff_dialogue("reset des variables.....");
		 fflush(stdout); 
                 for (i=0; i<26; i++) table[i]=0 ;
		 $$.text="reset all" ;
		 aff_les_variables();
                 sleep(2);
		 prompteur();
		}
       |  STOP FINDELIGNE
                {
		 $$.text="fin" ;
		 aff_dialogue(" Au revoir\n");

                YYACCEPT ;}
       ;

exp    :  exp '+' exp
                             {
			       $$.val = $1.val + $3.val ;
			       DECOMP_EXP2($$, $1, "+", $3);
			     }
       |  exp '-' exp
		             {
			       $$.val= $1.val - $3.val ;
			       DECOMP_EXP2($$, $1, "-", $3);
			     }
       |  exp '*' exp
		             {
			       $$.val = $1.val* $3.val ;
			       DECOMP_EXP2($$, $1, "*", $3);
			     }
       |  exp '/' exp
                             { if ($3.val == 00) {
	                             Mon_yyerror("division par zero");
			             $$.val = 999999;
			          } else
			          $$.val = $1.val / $3.val ;
			          DECOMP_EXP2($$, $1, "/", $3);
			     }
       | exp '^' exp
                        	{
				  $$.val = pow($1.val, $3.val) ;
				  DECOMP_EXP2($$, $1, "^", $3);
				}
       | '(' exp ')'
                        	{$$=$2;
			       DECOMP_EFUNC($$, "", $2 );
				}
       | '-' exp %prec UMOINS
                        	{$$.val = -$2.val ; $$.text=$2.text;
			       DECOMP_EXP1($$, "-", $2);
				}
       | ENT
                                {
				  $$.val = $1.val ;
				  $$.text = (char *)malloc(32);
				  sprintf($$.text, "%d", (int)$$.val);
				}
      | DERNIER
                                {
				  $$.val = dernier ;
				  $$.text = (char *)malloc(2);
				  strcpy($$.text, "$");
				}
       | PI
                                {
				  $$.val = 3.14159 ;
				  $$.text = (char *)malloc(2 + 1);
				  strcpy($$.text, "PI");
				}
       | VAR
                                {
				  $$.val = table[$1.let-'a'] ;

				  $$.text = (char *)malloc(2);
				  $$.text[0] = $1.let;
				  $$.text[1] = '\0';
				}
       | '(' cond '?' exp ':' exp  ')'
                        	{
				  $$.val = ($2.val==0)?$4.val:$6.val;
				  DECOMP_INST3($$, "(", $2 , "?", $4 , ":", $6 , ")");
			    }
       | '(' FROM exp TO exp SUM exp ')'
                        	{
				  S=0;
				  for (i= $3.val ; i<=$5.val ; i+=1) S+= $7.val;
				  $$.val = S;
				  DECOMP_INST3($$, "(from", $3 , "to", $5 , "sum", $7 , ")");
				}
       |  '(' FROM exp TO exp BY exp SUM exp ')'
                          {
			    S=0;
			    for (i= $3.val ; i<=$5.val ; i+=$7.val) S+= $9.val;
			    $$.val = S;
			    DECOMP_INST4($$, "(from", $3 , "to", $5 , "by", $7 , "sum", $9 , ")");
			  }
       | SINUS '(' exp ')'
                          {
			    $$.val = Eval("sin", $3.val );
			    DECOMP_EFUNC($$, "sinus", $3 );
			  }
       | COSINUS '(' exp ')'
                          {
			    $$.val = Eval("cos", $3.val );
			    DECOMP_EFUNC($$, "cosinus", $3 );
			  }
       ;


cond   :  exp '>' exp
		             {
			       $$.val = ($1.val > $3.val)? 0:1;
			       DECOMP_EXP2($$, $1, ">", $3);
			     }
       |  exp '>' '=' exp
		             {
			       $$.val = ($1.val >= $4.val)? 0:1;
			       DECOMP_EXP2($$, $1, ">=", $4);
			     }
       |  exp '<' exp
		             {
			       $$.val = ($1.val <  $3.val)? 0:1;
			       DECOMP_EXP2($$, $1, "<", $3);
			    }
       |  exp '<' '=' exp
		             {
			       $$.val = ($1.val <= $4.val)? 0:1;
			       DECOMP_EXP2($$, $1, "<=", $4);
			    }
       |  exp '=' '=' exp
		             {
			       $$.val = ($1.val == $4.val)? 0:1;
			       DECOMP_EXP2($$, $1, "==", $4);
			    }
       |  exp '!' '=' exp
		             {
			       $$.val = ($1.val != $4.val)? 0:1;
			       DECOMP_EXP2($$, $1, "!=", $4);
			    }
       ;
%%

extern int lig;

int yyparse();

int yyerror(char *s)   /* pour masquer les messages de yacc */
{
  /* printf("ligne %d : %s\n",lig,s); */
  return 1;
}
int Mon_yyerror(char *s) /* pour masquer celui de yacc */
{
  aff_erreur(lig, s);
  return 1;
}

int main() {
  dessine_calculette();
  
  return yyparse();
}

