/*** CALCULETTE PROGRAMMABLE (style Maple):
   decompcalc.y
   Syntaxe LALR(1) sauf Exp
   Evaluation par un (triplet) attribut synthetise $
   auteur : PFZ
   version origine: Mars 1999 avec l'aide de J. Farre'
   nouvelle version: Mars 2000 avec l'aide de T. Graf
   nouvelle version 2001: 
 
   version actuelle:  15 NOV 2005
Versions et MODIFS pour la (super)Calculette
_______________________________________________

pour 2000:
- ajout de la dcompilation du source
- nouveau makefile
- nouvelle version de l'affichage et dialogue

pour 2001: 
- grammaire E+E avec priorit
- accepter les Var avec Majuscules (transposes en minuscules par lex)

pour 2002:
- utiliser la grammaire ETF + Primaire pour - unaire et ^ 
- type abstrait pour la memoire REGISTRES avec setV et getV
- revoir le rcupration d'erreurs, insuffisante pour les regles basses de la grammaire
- decompilation via HTML ?
- IHM via java ?
*********************************************************/

/* declarations YACC *************/
%union {          /* le triplet d'attributs YACC */
    struct A {
      double val ;
      char let ;
      char* text ;
    } att;
}
%token <att> ENT VAR DERNIER REEL
%token FINDELIGNE CLEAR RESET PAUSE MENU STOP SUM FROM TO BY SINUS COSINUS SQRT LN PI E CNP PP MM
%type <att> exp cond ligne calc error
%left '+' '-'
%left '*' '/'
%left '^' 
%left UMOINS
%left '!' 

%{
#include <math.h> /* ajouter -lm au link */
#include <stdio.h>
#include <string.h>
#include "schemas.h" /* SCHEMAS d"ATTRIBUTION POUR LES REGLES DE YACC ***/
#include "affichage.h"
#include "memoire.h"

extern char DERNLIGNE[100]; // le source de la dernire ligne entre
extern int lig; // le numero de la ligne (d'entre) suivante
int ERREUR=0; // flag pour diffrencier les affichages en cas d'erreur
char* ERR_CAUSE=";"; // la cause de l'erreur
char YYERR[10]; // pour rcuperer la fentre de Yacc aprs une erreur syntaxique

%}

/***** DEBUT des actions YACC ****/
%%
calc   : ligne FINDELIGNE
         {
	   if(ERREUR) { 
	                erreurStyle(); 
			aff_source(lig-1, $1.text, ERR_CAUSE);
			ERREUR=0; 
			Mon_yyerror (ERR_CAUSE);
			lig -= 1; sprintf(ERR_CAUSE,";");
	   }
	   else { okStyle(); aff_source(lig-1, $1.text, ERR_CAUSE); } 
	   prompteur(lig,">");
	 }

       | calc ligne FINDELIGNE
         {
	   if(ERREUR) { 
	                erreurStyle(); 
			aff_source(lig-1, $2.text, ERR_CAUSE);  
			ERREUR=0; 
			Mon_yyerror (ERR_CAUSE);
			lig -= 1; sprintf(ERR_CAUSE,";");
	   }
	   else { okStyle(); aff_source(lig-1, $2.text, ERR_CAUSE); }
	   prompteur(lig,">");
  
	 }	  
	 | error  FINDELIGNE
        {
	  yyerrok; 
			/**** pas ici car systmatique sur '\n'
			printf("erreur sur: %c\n", yychar);
			voir YYERR rcupr par yyerror()
			******/ 	   
			{
			ERREUR=1;
	  		ERR_CAUSE=(char*)malloc(25) ;
	  		sprintf(ERR_CAUSE," ..>[error on %s]", YYERR);
	  		////DECOMP_STR($$, DERNLIGNE) ;
	  		erreurStyle(); 
			aff_source(lig-1, DERNLIGNE, ERR_CAUSE);  
			Mon_yyerror (ERR_CAUSE);
			lig -= 1; 
			ERREUR=0;
			sprintf(ERR_CAUSE,";");
	  		prompteur(lig,">");
			}
		}
       ;

ligne  :     /* vide */
	 	{
	   ///erreurStyle(); 
	   ///aff_dialogue("?!?"); 
	   ///prompteur(lig,">");
	   ERREUR=1; 
	   ERR_CAUSE=(char*)malloc(5) ;
	   sprintf(ERR_CAUSE," ?!?");
	   DECOMP_STR($$, "") ;
		}
       |  exp 
		{		  
		  if(!ERREUR) {
		  		okStyle();
				aff_exp($1.val);
				Dernier=$1.val;
		  }
		  DECOMP_EXP1($$, "=", $1);
		}

       |  VAR '=' exp 
               { 
		 if(!ERREUR) {
		                okStyle();
                		aff_affect($1.let, $3.val); 
				aff_une_variable($1.let, $3.val);
                		setOneMemoire($1.let, $3.val);	
                		Dernier=$3.val;
		  	}
			DECOMP_AFF($$, $1, $3);
          }

       |  CLEAR VAR 
	      {
		DECOMP_VAR($$, "clear ", $2.let, "");  
		okStyle();
		aff_clear($2.let); 
		aff_une_variable($2.let, 0);
		clearOneMemoire($2.let); 
          }

       |  RESET 
	      {
		DECOMP_STR($$, "reset all") ;
		okStyle(); 
		aff_dialogue("reset des variables...");
		fflush(stdout); 
         	resetAllMemoire(); 
		aff_les_variables();
         	sleep(1);
		prompteur(lig,">");
		}

       |  MENU 
	      {
		DECOMP_STR($$, "menu") ;
		okStyle(); 
		aff_dialogue("menu .....");
		fflush(stdout); 
		aff_menu(1) ;
		// sleep(1);
		//prompteur(lig,">");
		}

       |  STOP 
        {
	  DECOMP_STR($$, "FIN");
	  okStyle(); 
	  aff_dialogue(" Au revoir"); 
	  aff_source(lig-1, $$.text, ERR_CAUSE) ;
	  YYACCEPT ;
		}

       |  PAUSE exp 
        {
	  DECOMP_INT($$, "pause ", (int)$2.val); 
	  sleep((int)$2.val);
		}
       ;

exp    :  exp '+' exp
                 {
		   $$.val = $1.val + $3.val ;
		   DECOMP_EXP2($$, $1, "+", $3);
			     }
       |  exp '-' exp
		 {
		   $$.val= $1.val - $3.val ;
		   DECOMP_EXP2($$, $1, "-", $3);
			     }
       |  exp '*' exp
		  {
		    $$.val = $1.val* $3.val ;
		    DECOMP_EXP2($$, $1, "*", $3);
			     }
       |  exp '/' exp
                 { if ($3.val == 00) { 
				      ERREUR=1; 
			     	      // Mon_yyerror("division par zero");
			              $$.val = 999999;
				      char* text=malloc(strlen($3.text)+6);
				      sprintf(text, " <<%s>> ", $3.text);
				      $3.text=text;
				      ERR_CAUSE=(char*)malloc(15) ;
				      sprintf(ERR_CAUSE," ..> DIV:/0");
			        } 
		    else $$.val = $1.val / $3.val ;
		    DECOMP_EXP2($$, $1, "/", $3);
		 }

       | exp '^' exp
                {
		  $$.val = pow($1.val, $3.val) ;
		  DECOMP_EXP2($$, $1, "^", $3);
			     }

       | '(' exp ')'
                {
		  $$=$2;
		  DECOMP_EFUNC($$, "", $2 );
			     }

       | '-' exp %prec UMOINS
                {
		  $$.val = -$2.val ; //$$.text=$2.text;
		  DECOMP_EXP1($$, "-", $2);
			     }

       | exp '!' 
                {
		  SX=1;
		  for (i= 1 ; i<=(int)$1.val ; i++) SX*= i;
		  $$.val = SX ; 
		  DECOMP_FAC($$, $1);
				}
       | CNP '(' exp ',' exp ')' 
                {if ($3.val < $5.val) { 
				      ERREUR=1; 
			     	      // Mon_yyerror("Cpn au lieu de Cnp");
			              $$.val = 999999;
				      char* text=malloc(strlen($3.text)+6);
				      sprintf(text, " <<%s>> ", $3.text);
				      $3.text=text;
				      ERR_CAUSE=(char*)malloc(15) ;
				      sprintf(ERR_CAUSE," ..> CNP:n<p");
			        } 
		else {    
		  SY=1;
		  for (i= 1 ; i<=(int)$5.val ; i++) SY*= i;
		  SZ=1;
		  for (i=(int)$3.val ; i>=((int)$3.val-(int)$5.val+1) ; i--) SZ*= i;
		  $$.val = SZ/SY ; 
		 }
		  DECOMP_CNP($$, $3, $5);
		}

       | ENT
                {
		  $$.val = $1.val ;
		  DECOMP_INT($$, "", (int)$1.val);
		}

       | REEL
                {
		  $$.val = $1.val ;
		  DECOMP_REEL($$, "", $1.val);
		}

       | DERNIER
                {
		  $$.val = Dernier ;
		  DECOMP_STR($$, "$");
		}
       | PI
               {
		 $$.val = 3.14159 ;
		 DECOMP_STR($$, "#PI");
	       }
       | E
               {
		 $$.val = 2.71828 ;
		 DECOMP_STR($$, "#e");
	       }				
				
       | VAR
               {
		 $$.val = getOneMemoire($1.let);	
		 DECOMP_VAR($$, "", $1.let, "");
	       }
	 | VAR PP
               {
		 $$.val = getOneMemoire($1.let);
		 setOneMemoire($1.let, $$.val+1);
		 okStyle();
		 Dernier=$$.val+1;
             aff_affect($1.let, Dernier); 
		 aff_une_variable($1.let, Dernier);   	         	
		 DECOMP_VAR($$, "", $1.let, "++");
	       }
	 | VAR MM
               {
		 $$.val = getOneMemoire($1.let);
		 setOneMemoire($1.let, $$.val-1);
		 okStyle();
		 Dernier=$$.val-1;
             aff_affect($1.let, Dernier); 
		 aff_une_variable($1.let, Dernier);   	     	
		 DECOMP_VAR($$, "", $1.let, "--");
	       }	 
	| PP VAR
               {
		 $$.val = 1+getOneMemoire($2.let);
		 setOneMemoire($2.let, $$.val);
		 okStyle();
             aff_affect($2.let, $$.val); 
		 aff_une_variable($2.let, $$.val);   	
             Dernier=$$.val;			
		 DECOMP_VAR($$, "++", $2.let, "");
	       }	
	| MM VAR
               {
		 $$.val = getOneMemoire($2.let)-1;
		 setOneMemoire($2.let, $$.val);
		 okStyle();
             aff_affect($2.let, $$.val); 
		 aff_une_variable($2.let, $$.val);   	
             Dernier=$$.val;	
		 DECOMP_VAR($$, "--", $2.let, "");
	       }			
       | '(' cond '?' exp ':' exp  ')'
                {
		  $$.val = ($2.val==0)?$4.val:$6.val;
		  DECOMP_INST3($$, "(", $2 , "?", $4 , ":", $6 , ")");
			    }
				
       | '(' FROM exp TO exp SUM exp ')'
                {
		  SX=0.0;
		  for (i= $3.val ; i<=$5.val ; i+=1) SX+= $7.val;
		  $$.val = SX;
		  DECOMP_INST3($$, "(from", $3 , "to", $5 , "sum", $7 , ")");
				}
				
       |  '(' FROM exp TO exp BY exp SUM exp ')'
              {
		SX=0.0;
		for (i= $3.val ; i<=$5.val ; i+=$7.val) SX+= $9.val;
		$$.val = SX;
		DECOMP_INST4($$, "(from", $3 , "to", $5 , "by", $7 , "sum", $9 , ")");
			  }
			  
       | SINUS '(' exp ')'
			  {
			    $$.val = Eval("sin", $3.val );
			    DECOMP_EFUNC($$, "sinus", $3 );
			  }
			  
       | COSINUS '(' exp ')'
              {
			    $$.val = Eval("cos", $3.val );
			    DECOMP_EFUNC($$, "cosinus", $3 );
			  }
			  
       | SQRT '(' exp ')'
              {
			  	if ($3.val < 00) {
					ERREUR=1; 
			  		//Mon_yyerror("Racine: Op<0");
					$$.val = 999999;
					char* text=malloc(strlen($3.text)+6);
					sprintf(text, " <<%s>> ", $3.text);
					$3.text=text;
					ERR_CAUSE=(char*)malloc(20) ;
					sprintf(ERR_CAUSE, " ..>SQRT:Op<0");
			     	} 
			    else $$.val = Eval("sqrt", $3.val );
			    DECOMP_EFUNC($$, "sqrt", $3 );
			  }
			  
       | LN '(' exp ')'
              {
			  	if ($3.val <= 00) {
					ERREUR=1; 
			  		//Mon_yyerror("Ln: Op<=0");
					$$.val = 999999;
					char* text=malloc(strlen($3.text)+6);
					sprintf(text, " <<%s>> ", $3.text);
					$3.text=text;
					ERR_CAUSE=(char*)malloc(20) ;
					sprintf(ERR_CAUSE, " ..>LN:Op<=0");
			        } 
			   else $$.val = Eval("ln", $3.val );
			   DECOMP_EFUNC($$, "ln", $3 );
			  }

       ;


cond   :  exp '>' exp
		       {
			       $$.val = ($1.val > $3.val)? 0:1;
			       DECOMP_EXP2($$, $1, ">", $3);
			     }
				 
       |  exp '>' '=' exp
		       {
			       $$.val = ($1.val >= $4.val)? 0:1;
			       DECOMP_EXP2($$, $1, ">=", $4);
			     }
				 
       |  exp '<' exp
		       {
			       $$.val = ($1.val <  $3.val)? 0:1;
			       DECOMP_EXP2($$, $1, "<", $3);
			    }
				
       |  exp '<' '=' exp
		       {
			       $$.val = ($1.val <= $4.val)? 0:1;
			       DECOMP_EXP2($$, $1, "<=", $4);
			    }
				
       |  exp '=' '=' exp
		       {
			       $$.val = ($1.val == $4.val)? 0:1;
			       DECOMP_EXP2($$, $1, "==", $4);
			    }
				
       |  exp '#' exp
		      {
			       $$.val = ($1.val != $3.val)? 0:1;
			       DECOMP_EXP2($$, $1, "!=", $3);
			    }
				
       ;
%%
int yyparse();

int yyerror(char *s) {  // pour masquer les messages de yacc 
  /* c'est ci la bonne place pour rcuper la fentre qui provoque l'erreur syntaxique */
  
/***** extrait de dparse.h pour les TOKEN de YYERR *********/  
#define NBTOKEN 22 
const char* TOKENS[NBTOKEN] = {  // index -257
"#ENT  ",   //257
"#VAR  ",   //258
"#DER  ",   //259
"#REEL ",   //260
"#FDL  ",   //261
"#CLEAR",   //262
"#RESET",   //263
"#PAUSE",   //264
"#MENU ",   //265
"#EXIT ",   //266
"#SUM  ",   //267
"#FROM ",   //268
"#TO   ",   //269
"#BY   ",   //270
"#SIN  ",   //271
"#COS  ",   //272
"#SQRT ",   //273
"#LOG  ",   //274
"#PI   ",  //275
"#e    ",  //276
"#CNP  ",  //277
"#UMOIN"   //278 
};
 
  if (yychar>=32 & yychar<=128) sprintf(YYERR, "%c", yychar);
  else 
  sprintf(YYERR, "%s", TOKENS[yychar-257]);
	
  /* printf("ligne %d : %s\n",lig,s); */
  return 1;
}

int Mon_yyerror(char *s) { 
  aff_erreur(lig, s);
  return 1;
}

int main() {
  dessine_calculette();
  okStyle();
  aff_dialogue("Bienvenue sur SuperCalc");
  prompteur(lig,">");
  yyparse();
  auRevoir ("et  Bientt ...\n");
}

/******************************** FIN *********************/
